package com.nisum;

import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.powermock.core.classloader.annotations.PowerMockIgnore;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;
import java.io.IOException;
import static org.mockito.Mockito.*;


@RunWith(PowerMockRunner.class)
@PowerMockIgnore("jdk.internal.reflect.*")
@PrepareForTest({HateoasApplication.class,JSONObject.class})
public class HateoasApplicationTests {

	@InjectMocks
	HateoasApplication hateoasApplication;

	@Mock
	public JSONParser parser;

	@Before()
	public void setUp() throws Exception {
		MockitoAnnotations.initMocks(this);
		JSONObject mock = Mockito.mock(JSONObject.class);
		when(hateoasApplication.getRestTemplate().getForObject(anyString(),String.class)).thenReturn(anyString());
		when(parser.parse(anyString())).thenReturn(mock);
		when(hateoasApplication.getJSONObject(anyString())).thenReturn(mock);
	}

	@Test
	public void getUsingRestTemplateTest() throws IOException, ParseException {
		hateoasApplication.getUsingRestTemplate();
	}

	@Test
	public void getUsingHttpClientTest() throws IOException, ParseException {
		DefaultHttpClient httpClient = mock(DefaultHttpClient.class);
		HttpGet getRequest = mock(HttpGet.class);
		HttpResponse httpResponse = mock(HttpResponse.class);
		StatusLine statusLine = mock(StatusLine.class);
		when(statusLine.getStatusCode()).thenReturn(200);
		when(httpResponse.getStatusLine()).thenReturn(statusLine);
		when(httpClient.execute(getRequest)).thenReturn((CloseableHttpResponse) httpResponse);
		Assert.assertEquals(statusLine.getStatusCode(),200);
		hateoasApplication.getUsingHttpClient();
	}

	@Test
	public void getUsingHttpClientExceptionTest() throws IOException, ParseException {
		HttpClient httpClient = mock(HttpClient.class);
		HttpGet httpGet = mock(HttpGet.class);
		StatusLine statusLine = mock(StatusLine.class);
		when(httpClient.execute(httpGet)).thenThrow(RuntimeException.class);
		Assert.assertEquals(statusLine.getStatusCode(),0);
		hateoasApplication.getUsingHttpClient();
	}

	@Test
	public void getJSONObjectTest() throws ParseException, IOException {
		verify(hateoasApplication.getJSONObject(anyString()), times(4));
	}

	@Test
	public void getJSONObjectHttpTest() throws ParseException, IOException {
		verify(hateoasApplication.getJsonObjectHttp(any(HttpResponse.class),anyInt()), times(4));
	}

}
